﻿/*
 * 
 * Copyright (c) 2016, H.R.Z. Software services LTD
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions 
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer 
 *    in the documentation and/or other materials provided with the distribution.
 * 
 * 3. Neither the name of the copyright holder nor the names of its contributors may be used to endorse or promote products derived from 
 *    this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, 
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, 
 * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Runtime.InteropServices;
using System.Diagnostics;
using System.IO; 
using rzdcxLib;

namespace DIHOFA
{
    class Program
    {
        static private List<string> FilesLists = new List<string>();

        static private string FailedFolderPath;
 
        static void Main(string[] args)
        {
            try
            {
                //If previous sending process was not finished don't start new one
                Process[] pros = Process.GetProcessesByName("DIHOFA");
                if ((pros != null) && (pros.Length > 1))
                {
                    EventLogger.LogEvent(System.Diagnostics.EventLogEntryType.Warning, "Previous sending was not finished");
                    return;
                }

                if (Directory.Exists(Properties.Settings.Default.DIHOFA_Hot_Folder))
                {
                    Program.ProcessNewFiles();
                }
                else
                {
                    EventLogger.LogEvent(System.Diagnostics.EventLogEntryType.Error, "Hot folder '" + Properties.Settings.Default.DIHOFA_Hot_Folder + "' doesn't exist");
                }
            }
            catch (Exception ex)
            {
                EventLogger.LogEvent(System.Diagnostics.EventLogEntryType.Error, ex.Message);
            }
        }

        /// <summary>
        /// Scan hot folder for new files and send them to the PACS
        /// </summary>
        static private void ProcessNewFiles()
        {
            DCXREQ req = null;
            string succeededInstances;
            string failedInstances;

            Program.FailedFolderPath = Properties.Settings.Default.Failed_Files_Folder;
            if (Program.FailedFolderPath[Program.FailedFolderPath.Length - 1] != '\\')
            {
                Program.FailedFolderPath += @"\";
            }

            Program.BuildListsOfFilesToSend(Properties.Settings.Default.DIHOFA_Hot_Folder);

            if (Program.FilesLists.Count > 0)
            {
                foreach (string FilesListLine in Program.FilesLists)
                {
                    try
                    {
                        req = new DCXREQ();
                        req.Send(Properties.Settings.Default.Application_AE_Title,
                                 Properties.Settings.Default.Target_AE_Title,
                                 Properties.Settings.Default.Target_Host,
                                 Properties.Settings.Default.Target_Port,
                                 FilesListLine,
                                 out succeededInstances,
                                 out failedInstances);

                        Program.HandleProccessedFiles(succeededInstances, failedInstances);
                        Program.EraseEmptyFolders(Properties.Settings.Default.DIHOFA_Hot_Folder);

                        if (succeededInstances.Length > 0)
                        {
                            EventLogger.LogEvent(System.Diagnostics.EventLogEntryType.Information,
                                                 " Sent: " + succeededInstances);
                        }

                        if (failedInstances.Length > 0)
                        {
                            EventLogger.LogEvent(System.Diagnostics.EventLogEntryType.Error,
                                                 " Failed: " + failedInstances);
                        }
                    }
                    catch (Exception e)
                    {
                        EventLogger.LogEvent(System.Diagnostics.EventLogEntryType.Error, e.Message);
                    }
                    finally
                    {
                        Program.ReleaseComObject(req);
                    }
                }
            }
        }

        /// <summary>
        /// Build list of "To send" lines (each line is "[file1];[file2];...")
        /// </summary>
        /// <returns></returns>
        static void BuildListsOfFilesToSend(string DirPath)
        {
            string[] files = Directory.GetFiles(DirPath, "*.*", SearchOption.AllDirectories);

            int Counter = 0;
            string CurrLine = String.Empty; 
            foreach (string s in files)
            {
                if (Counter == Properties.Settings.Default.Maximum_Files_To_Send)
                {
                    //Current line filled, add it to lines list
                    Program.FilesLists.Add(CurrLine);
                    CurrLine = String.Empty;
                    Counter = 0;
                }
                CurrLine += (s + ";");
                Counter++;
            }

            //Add last line to lines list
            if (CurrLine.Length > 0)
                Program.FilesLists.Add(CurrLine);

        }

        /// <summary>
        /// Remove files which were sent, move failed files to "Failed" folder
        /// </summary>
        /// <param name="succeededInstances"></param>
        /// <param name="failedInstances"></param>
        static void HandleProccessedFiles(string succeededInstances, string failedInstances)
        {
            //Remove sent files if needed
            if (Properties.Settings.Default.EraseSentFiles)
            {
                string[] files = succeededInstances.Split(';');
                foreach (string s in files)
                {
                    if (s.Trim() != String.Empty)
                    {
                        File.Delete(s);
                    }
                }
            }

            //Move failed files to "Failed" folder if needed
            if (Properties.Settings.Default.MoveFailedFiles)
            {
                if (!Directory.Exists(Program.FailedFolderPath))
                {
                    Directory.CreateDirectory(Program.FailedFolderPath);
                }
                string[] dirs = failedInstances.Split(';');
                string MoveToName;
                string MoveToDir = String.Empty;
                FileInfo fi;
                string FailedFileRelativePath;
                string FailedFileName;
                foreach (string s in dirs)
                {
                    if (s.Trim() != String.Empty)
                    {
                        fi = new FileInfo(s); 
                        FailedFileRelativePath = fi.DirectoryName.Replace(Properties.Settings.Default.DIHOFA_Hot_Folder, @"\");
                        FailedFileName = fi.Name;
                        MoveToName = Program.FailedFolderPath + FailedFileRelativePath + @"\" + FailedFileName;

                        //"Move" can not overwrite so we need to delete existing file
                        //to avoid exception
                        if (File.Exists(MoveToName))
                        {
                            File.Delete(MoveToName);
                        }
                        
                        //There is no option to create file with path hierarchy on move (or copy)
                        //so we need to create path manually in the "Failed" folder
                        //Don't check the same path more than one time
                        if (MoveToDir != Program.FailedFolderPath + FailedFileRelativePath)
                        {
                            MoveToDir = Program.FailedFolderPath + FailedFileRelativePath;
                            //If "Move to" folder doesn't exist create it
                            if (!Directory.Exists(MoveToDir))
                            {
                                Directory.CreateDirectory(MoveToDir);
                            }
                        }
                        File.Move(s, MoveToName);
                    }
                }
            }
        }

        /// <summary>
        /// Erase all empty sub-folders from DIHOFA hot folder
        /// </summary>
        /// <returns></returns>
        static void EraseEmptyFolders(string DirPath)
        {
            if(Directory.GetFiles(DirPath).Length > 0)
            {
                //Folder contains files
                return;
            }
            
            string[] dirs = Directory.GetDirectories(DirPath);
            if (dirs.Length > 0)
            {
                //Folder contains sub-folders that we need to check
                foreach (string s in dirs)
                {
                    //Not check and erase "Failed" folder
                    if (!Program.FailedFolderPath.Contains(s))
                    {
                        Program.EraseEmptyFolders(s);
                    }
                }
            }

            //Check whether folder contains any sub-folder now
            //(after recursive erasing or if last level sub-folder is empty)
            //Not root folder folder!!!
            if (DirPath != Properties.Settings.Default.DIHOFA_Hot_Folder)
            {
                if (Directory.GetDirectories(DirPath).Length == 0)
                {
                    Directory.Delete(DirPath);  
                }
            }
        }

        /* This has to be called for all the com objects to release the memory!!! */
        static private void ReleaseComObject(object o)
        {
            if (o != null)
                Marshal.ReleaseComObject(o);
        }
    }
}
